@@ -1,3 +1,6 @@
+__all__ = ['select_interface', 'BaseInterface', 'PropertyName', 'LoopStatus', 'PlaybackPropertyName',
+           'PlaybackProperties', 'PlaybackStatus', 'NowPlayingInterface']
+
 from typing import Type
 
 from aionowplaying.interface import select_interface, BaseInterface
