@@ -1,24 +1,33 @@
 import asyncio
+import threading
 from typing import Any
+from datetime import timedelta
 
-from winrt.windows.foundation import Uri, TimeSpan
-from winrt.windows.media import SystemMediaTransportControlsTimelineProperties, SystemMediaTransportControls, \
+from winsdk.windows.foundation import Uri
+from winsdk.windows.foundation.collections import IVector
+from winsdk.windows.media import SystemMediaTransportControlsTimelineProperties, SystemMediaTransportControls, \
     SystemMediaTransportControlsDisplayUpdater, MediaPlaybackStatus, MediaPlaybackType, MediaPlaybackAutoRepeatMode, \
     AutoRepeatModeChangeRequestedEventArgs, SystemMediaTransportControlsButtonPressedEventArgs, \
     SystemMediaTransportControlsButton, PlaybackPositionChangeRequestedEventArgs, PlaybackRateChangeRequestedEventArgs, \
     SystemMediaTransportControlsPropertyChangedEventArgs, SystemMediaTransportControlsProperty, \
     ShuffleEnabledChangeRequestedEventArgs
-from winrt.windows.media.playback import MediaPlayer
-from winrt.windows.storage.streams import RandomAccessStreamReference
+from winsdk.windows.media.playback import MediaPlayer
+from winsdk.windows.storage.streams import RandomAccessStreamReference
 
 from aionowplaying import BaseInterface, PropertyName, PlaybackPropertyName
 from aionowplaying.interface.base import TrackListPropertyName, PlaybackStatus, PlaybackProperties, LoopStatus, \
     MediaType
 
 
+def TimeSpan(x):
+    return timedelta(microseconds=x/10000)
+
+
 class WindowsInterface(BaseInterface):
     def __init__(self, name):
         super(WindowsInterface, self).__init__(name)
+        self._loop = asyncio.get_event_loop()
+        self._running = True
         self._playback_properties = PlaybackProperties()
         self._player = MediaPlayer()
         self._controls: SystemMediaTransportControls = self._player.system_media_transport_controls
@@ -33,37 +42,62 @@ class WindowsInterface(BaseInterface):
 
     def shuffle_change_requested(self, _, args: ShuffleEnabledChangeRequestedEventArgs):
         shuffle_enabled: bool = args.requested_shuffle_enabled
-        asyncio.run(self.on_shuffle(shuffle_enabled))
+        if asyncio.iscoroutinefunction(self.on_shuffle):
+            self._run_task(self.on_shuffle(shuffle_enabled))
+        else:
+            self.on_shuffle(shuffle_enabled)
 
     def property_changed(self, _, args: SystemMediaTransportControlsPropertyChangedEventArgs):
         property_: SystemMediaTransportControlsProperty = args.property
         if property_ == SystemMediaTransportControlsProperty.SOUND_LEVEL:
-            asyncio.run(self.on_volume(self._controls.sound_level))
+            if asyncio.iscoroutinefunction(self.on_volume):
+                self._run_task(self.on_volume(self._controls.sound_level))
+            else:
+                self.on_volume(self._controls.sound_level)
 
     def playback_rate_change_requested(self, _, args: PlaybackRateChangeRequestedEventArgs):
         rate: float = args.requested_playback_rate
-        asyncio.run(self.on_rate(rate))
+        if asyncio.iscoroutinefunction(self.on_rate):
+            self._run_task(self.on_rate(rate))
+        else:
+            self.on_rate(rate)
 
     def playback_position_change_requested(self, _, args: PlaybackPositionChangeRequestedEventArgs):
-        position: TimeSpan = args.requested_playback_position
-        print(position, dir(position))
+        position = args.requested_playback_position
         if self._playback_properties.CanSeek:
-            asyncio.run(self.on_set_position(self._playback_properties.Metadata.id_, position.duration))
+            if asyncio.iscoroutinefunction(self.on_set_position):
+                self._run_task(self.on_set_position(self._playback_properties.Metadata.id_, int(position.duration / 10000)))
+                self._run_task(self.on_seek(int(position.duration / 10000)))
+            else:
+                self.on_set_position(self._playback_properties.Metadata.id_, int(position.duration / 10000))
+                self.on_seek(int(position.duration / 10000))
 
     def button_pressed(self, _, args: SystemMediaTransportControlsButtonPressedEventArgs):
         button: SystemMediaTransportControlsButton = args.button
         if button == SystemMediaTransportControlsButton.PLAY and self._playback_properties.CanPlay:
-            asyncio.run(self.on_play())
+            if asyncio.iscoroutinefunction(self.on_play):
+                self._run_task(self.on_play())
+            else:
+                self.on_play()
             self._controls.playback_status = MediaPlaybackStatus.PLAYING
             self._playback_properties.PlaybackStatus = PlaybackStatus.Playing
         if button == SystemMediaTransportControlsButton.PAUSE and self._playback_properties.CanPause:
-            asyncio.run(self.on_pause())
+            if asyncio.iscoroutinefunction(self.on_pause):
+                self._run_task(self.on_pause())
+            else:
+                self.on_pause()
             self._controls.playback_status = MediaPlaybackStatus.PAUSED
             self._playback_properties.PlaybackStatus = PlaybackStatus.Paused
         if button == SystemMediaTransportControlsButton.NEXT and self._playback_properties.CanGoNext:
-            asyncio.run(self.on_next())
+            if asyncio.iscoroutinefunction(self.on_next):
+                self._run_task(self.on_next())
+            else:
+                self.on_next()
         if button == SystemMediaTransportControlsButton.PREVIOUS and self._playback_properties.CanGoPrevious:
-            asyncio.run(self.on_previous())
+            if asyncio.iscoroutinefunction(self.on_previous):
+                self._run_task(self.on_previous())
+            else:
+                self.on_previous()
 
     def auto_repeat_mode_change_requested(self, _, args: AutoRepeatModeChangeRequestedEventArgs):
         value = LoopStatus.None_
@@ -72,7 +106,10 @@ class WindowsInterface(BaseInterface):
             value = LoopStatus.Playlist
         elif mode == MediaPlaybackAutoRepeatMode.TRACK:
             value = LoopStatus.Track
-        asyncio.run(self.on_loop_status(value))
+        if asyncio.iscoroutinefunction(self.on_loop_status):
+            self._run_task(self.on_loop_status(value))
+        else:
+            self.on_loop_status(value)
         self._playback_properties.LoopStatus = value
 
     def set_property(self, name: PropertyName, value: Any):
@@ -103,7 +140,7 @@ class WindowsInterface(BaseInterface):
             self._update_metadata(value)
             self._playback_properties.Metadata = value
         elif name == PlaybackPropertyName.Shuffle:
-            self._controls.is_shuffle_enabled = value
+            self._controls.shuffle_enabled = value
             self._playback_properties.Shuffle = value
         elif name == PlaybackPropertyName.Rate:
             self._controls.playback_rate = value
@@ -117,7 +154,7 @@ class WindowsInterface(BaseInterface):
                 self._controls.auto_repeat_mode = MediaPlaybackAutoRepeatMode.TRACK
             self._playback_properties.LoopStatus = value
         elif name == PlaybackPropertyName.Position:
-            self._timeline.position = TimeSpan(value)
+            self._timeline.position = TimeSpan(value * 10000)
             self._controls.update_timeline_properties(self._timeline)
 
     def _update_metadata(self, value: PlaybackProperties.MetadataBean):
@@ -125,21 +162,23 @@ class WindowsInterface(BaseInterface):
         if value.media_type == MediaType.Image:
             self._updater.type = MediaPlaybackType.IMAGE
         elif value.media_type == MediaType.Video:
-            self._updater.type = MediaPlaybackType.Video
+            self._updater.type = MediaPlaybackType.VIDEO
         else:
             self._updater.type = MediaPlaybackType.MUSIC
         self._updater.app_media_id = value.id_
         self._updater.music_properties.artist = ','.join(value.artist)
         self._updater.music_properties.title = value.title
         self._updater.music_properties.album_title = value.album
-        self._updater.music_properties.genres = value.genre
-        self._updater.thumbnail = RandomAccessStreamReference.create_from_uri(Uri(value.url))
+        self._updater.music_properties.genres: IVector
+        self._updater.music_properties.genres.replace_all(value.genre)
+        if value.cover:  # not None and not empty
+            self._updater.thumbnail = RandomAccessStreamReference.create_from_uri(Uri(value.cover))
         self._updater.update()
         # update timeline
         self._timeline.start_time = TimeSpan(0)
-        self._timeline.end_time = TimeSpan(value.duration)
+        self._timeline.end_time = TimeSpan(value.duration * 10000)
         self._timeline.min_seek_time = TimeSpan(0)
-        self._timeline.max_seek_time = TimeSpan(value.duration)
+        self._timeline.max_seek_time = TimeSpan(value.duration * 10000)
         self._controls.update_timeline_properties(self._timeline)
 
     def set_tracklist_property(self, name: TrackListPropertyName, value: Any):
@@ -149,5 +188,21 @@ class WindowsInterface(BaseInterface):
         return getattr(self._playback_properties, name.value)
 
     async def start(self):
-        while True:
+        # Don't need a background server for Windows
+        while self._running:
             await asyncio.sleep(1)
+        try:
+            self._player.close()
+        except:
+            pass
+
+    async def stop(self):
+        self._running = False
+
+    def _run_task(self, task):
+        # Windows callbacks may be invoked in non-main thread, besides,
+        # they may run in different threads.
+        if threading.current_thread() is not threading.main_thread():
+            asyncio.run_coroutine_threadsafe(task, self._loop)
+        else:
+            asyncio.create_task(task)
